#include <stdlib.h>
#include <iostream>
#include <ctime>
#include <cmath>
#include <cv.h>
#include <highgui.h>

#define forsn(i,s,n) for(i=s;i<n;++i)
#define forn(i,n) forsn(i,0,n)

using namespace std;

void cvPrint(char* str, IplImage* img){
	CvFont font;
	double hScale=1.0;
	double vScale=1.0;
	int    lineWidth=2;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, hScale,vScale,0,lineWidth);
	cvPutText (img,str,cvPoint(40,40), &font, cvScalar(255,255,0));
}

uchar sat(int x){
	if(x<0) return 0;
	else if (x>255) return 255;
	uchar t = x;
	return t;
}

void grayScale(IplImage* src, IplImage* out, uint *histogram){
	int height,width,step,outStep,channels;
	uchar *data, *srcd;
	int i,j, tmp;

	// get the image data
	height    = src->height;
	width     = src->width;
	step      = src->widthStep;
	channels  = src->nChannels;
	srcd      = (uchar *)src->imageData;

	forn(i,255) histogram[i] = 0;
	outStep   = out->widthStep;
	data = (uchar *)out->imageData;
	// convolve the image
	forn(i,height){ 
		forn(j,width){
			tmp = srcd[i*step+j*channels+2] << 1;	 //red
			tmp = (int) (((int)srcd[i*step+j*channels+1]) << 2) + (int)srcd[i*step+j*channels+1];	 //green
			tmp += srcd[i*step+j*channels+0];	 //blue
			data [i*outStep+j] = (unsigned char) (tmp>>3);
			histogram[(unsigned char) (tmp>>3)]++;
		}
	}
	return;
}

void histogramEq(IplImage *src, uint *histogram){
	int height,width,step,pixCount;
	uchar *srcd;
	int i,j;

	// get the image data
	height    = src->height;
	width     = src->width;
	pixCount  = height * width; 
	step      = src->widthStep;
	srcd      = (uchar *)src->imageData;

	uint sum = 0;
	uint lut[256];
	forn(i, 256){
		sum += histogram[i];
		lut[i] = sum * 255 / pixCount;
	}
	forn(i,height){ 
		forn(j,width){
			srcd[i*step+j] = lut[srcd[i*step+j]];
		}
	}
	return;

}

int main( int argc, char** argv )
{
	time_t start,end;
	IplImage  *frame;
	int       key, counter ;
	/* supply the AVI file to play */
	assert( argc == 2 );

	/* load the AVI file */
	CvCapture *capture = cvCaptureFromAVI( argv[1] );

	/* always check */
	if( !capture ) return 1;    

	/* get fps, needed to set the delay */
	int fps = ( int )cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	/* display video */
	cvNamedWindow( "inputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "inputVideo", 10, 10);

	cvNamedWindow( "outputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "outputVideo", 1000, 10);

	time(&start);
	counter = 0;

	frame = cvQueryFrame( capture );
	IplImage *outFrame = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);

	uint histogram[256];

	while( key != 1048689){ //'q' 
		frame = cvQueryFrame( capture );

		if( !frame ) break;

		grayScale(frame, outFrame, histogram);
		histogramEq(outFrame, histogram);

		/* display frame */
		cvShowImage( "inputVideo", frame );
		cvShowImage( "outputVideo", outFrame );

		time(&end);
		++counter;
		double sec = difftime(end,start);
		double fpsRatio = counter/sec;
		cout << fpsRatio << "\t fps " << endl;
		/* quit if user press 'q' */
		key = cvWaitKey( 1000 / fps);
	}

	/* free memory */
	cvReleaseCapture( &capture );
	cvDestroyWindow( "video" );

	return 0;
}
