#define BLOCK_SIZE 16	//debe coincidir con el BLOCK_SIZE del cpp
#define CHANNELS 3
#define M00 0
#define M01 1
#define M02 0
#define M10 1
#define M11 1
#define M12 1
#define M20 0
#define M21 1
#define M22 0
#define uchar unsigned char

unsigned char togray(uchar r, uchar g, uchar b, int threshold){
	int tmp;
	tmp = r << 1;	 //red
	tmp = (int) (((int)g << 2)) + (int)g;	 //green
	tmp += b;	 //blue
	tmp = (unsigned char) (tmp>>3);
	if(tmp >= threshold) return 255;
	else return 0;
}

__kernel void erode(
	const __global unsigned char *img,
	__global unsigned char *out, 
	const unsigned int width,
	const unsigned int height,
	const unsigned int step,
	const unsigned int outStep,
	const int threshold)
{
	
	int ijAcum, temp;	
	int j  = get_local_id(0);	//indice x dentro del bloque
	int i  = get_local_id(1);	//indice y dentro del bloque

	int gidj  = get_global_id(0);	//indice x global
	int gidi  = get_global_id(1);	//indice y global

	if(	gidj < width && 
		gidi < height )	//procesamos dentro de la imagen
	{
		//el tamanio es del bloque y los bordes
		__local unsigned char M[(BLOCK_SIZE*BLOCK_SIZE+4*BLOCK_SIZE+4)];
		int stepM = (BLOCK_SIZE+2);

		//el i+1 hace rol del central, el i del elem de arriba e i+2 del de abajo
		M[(i+1)*stepM+(j+1)] =  togray(	img[gidi*step+gidj*CHANNELS+2],
										img[gidi*step+gidj*CHANNELS+1],
										img[gidi*step+gidj*CHANNELS+0],
										threshold);
		//copia los bordes
		barrier(CLK_LOCAL_MEM_FENCE);
		if(i == 0 && gidi != 0) {
			M[(i)*stepM+(j+1)] = togray( 	img[(gidi-1)*step+gidj*CHANNELS+2],
											img[(gidi-1)*step+gidj*CHANNELS+1],
											img[(gidi-1)*step+gidj*CHANNELS+0],
											threshold);

		}
		if(i == BLOCK_SIZE-1 && gidi != height-1) {
			M[(i+2)*stepM+(j+1)] = togray(	img[(gidi+1)*step+gidj*CHANNELS+2],
											img[(gidi+1)*step+gidj*CHANNELS+1],
											img[(gidi+1)*step+gidj*CHANNELS+0],
											threshold);
		}
		if(j == 0 && gidj != 0) {
			M[(i+1)*stepM+(j)] = togray(	img[(gidi)*step+(gidj-1)*CHANNELS+2],
											img[(gidi)*step+(gidj-1)*CHANNELS+1],
											img[(gidi)*step+(gidj-1)*CHANNELS+0],
											threshold);
		}
		if(j == BLOCK_SIZE-1 && gidj != width-1) {
			M[(i+1)*stepM+(j+2)] = togray(	img[(gidi)*step+(gidj+1)*CHANNELS+2],
											img[(gidi)*step+(gidj+1)*CHANNELS+1],
											img[(gidi)*step+(gidj+1)*CHANNELS+0],
											threshold);
		}
		if(i == 0 && j == 0 && gidj != 0 && gidi != 0) {
			M[(i)*stepM+(j)] = togray(		img[(gidi-1)*step+(gidj-1)*CHANNELS+2],
											img[(gidi-1)*step+(gidj-1)*CHANNELS+1],
											img[(gidi-1)*step+(gidj-1)*CHANNELS+0],
											threshold);
		}
		if(i == 0 && j == BLOCK_SIZE-1 && gidj != width-1 && gidi != 0) {
			M[(i)*stepM+(j+2)] = togray(	img[(gidi-1)*step+(gidj+1)*CHANNELS+2],
											img[(gidi-1)*step+(gidj+1)*CHANNELS+1],
											img[(gidi-1)*step+(gidj+1)*CHANNELS+0],
											threshold);
		}
		if(i == BLOCK_SIZE-1 && j == BLOCK_SIZE-1 && gidj != width-1 && gidi != height-1) {
			M[(i+2)*stepM+(j+2)] = togray( 	img[(gidi+1)*step+(gidj+1)*CHANNELS+2],
											img[(gidi+1)*step+(gidj+1)*CHANNELS+1],
											img[(gidi+1)*step+(gidj+1)*CHANNELS+0],	
											threshold);
		}
		if(i == BLOCK_SIZE-1 && j == 0 && gidj != 0 && gidi != height-1) {
			M[(i+2)*stepM+(j)] = togray(	img[(gidi+1)*step+(gidj-1)*CHANNELS+2],
											img[(gidi+1)*step+(gidj-1)*CHANNELS+1],
											img[(gidi+1)*step+(gidj-1)*CHANNELS+0],
											threshold);
		}
		
		barrier(CLK_LOCAL_MEM_FENCE); // aseguramos que estan todos los datos
		for(int k = 0; k < CHANNELS; k++){
			ijAcum = M[(i)*stepM	+ (j)]	* M00;
			ijAcum += M[(i)*stepM	+ (j+1)]	* M01;
			ijAcum += M[(i)*stepM	+ (j+2)]	* M02;
			ijAcum += M[(i+1)*stepM	+ (j)]	* M10;
			ijAcum += M[(i+1)*stepM	+ (j+1)]	* M11;
			ijAcum += M[(i+1)*stepM	+ (j+2)]	* M12;
			ijAcum += M[(i+2)*stepM	+ (j)]	* M20;
			ijAcum += M[(i+2)*stepM	+ (j+1)]	* M21;
			ijAcum += M[(i+2)*stepM	+ (j+2)]	* M22;
			if(ijAcum < 0) ijAcum = 0;		//saturamos
			else if(ijAcum >= 255) ijAcum = 255;
			out[gidi*outStep+gidj] = ijAcum;	
		}
	}
}
