#include <stdlib.h>
#include <iostream>
#include <ctime>
#include <cmath>
#include <cv.h>
#include <highgui.h>

#define forsn(i,s,n) for(i=s;i<n;++i)
#define forn(i,n) forsn(i,0,n)

using namespace std;

void cvPrint(char* str, IplImage* img){
	CvFont font;
	double hScale=1.0;
	double vScale=1.0;
	int    lineWidth=2;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, hScale,vScale,0,lineWidth);
	cvPutText (img,str,cvPoint(40,40), &font, cvScalar(255,255,0));
}


uchar sat(int x){
	if(x<0) return 0;
	else if (x>255) return 255;
	uchar t = x;
	return t;
}

void grayScaleAndThreshold(IplImage* src, IplImage* out, uchar threshold){
	int height,width,step,outStep,channels;
	uchar *data, *srcd;
	int i,j, tmp;

	// get the image data
	height    = src->height;
	width     = src->width;
	step      = src->widthStep;
	channels  = src->nChannels;
	srcd      = (uchar *)src->imageData;

	outStep   = out->widthStep;
	data = (uchar *)out->imageData;
	// convolve the image
	forn(i,height){ 
		forn(j,width){
			tmp = srcd[i*step+j*channels+2] << 1;	 //red
			tmp = (int) (((int)srcd[i*step+j*channels+1]) << 2) + (int)srcd[i*step+j*channels+1];	 //green
			tmp += srcd[i*step+j*channels+0];	 //blue
			//data [i*outStep+j] = (unsigned char) (tmp>>3);
			tmp = (unsigned char) (tmp>>3);
			if(tmp >= threshold) data[i*outStep+j] = 255;
			else data[i*outStep+j] = 0;
		}
	}
	return;
}


IplImage* convolve(IplImage* src, int *m){	//la matriz m es de 3x3
	int height,width,step,channels;
	uchar *data, *srcd;
	int i,j;

	int acum;
	// get the image data
	height    = src->height;
	width     = src->width;
	step      = src->widthStep;
	channels  = src->nChannels;
	srcd      = (uchar *)src->imageData;

	IplImage *img = cvCreateImage(cvSize(width, height), src->depth, channels);
	cvCopy(src,img,NULL);

	data = (uchar *)img->imageData;
	// convolve the image
	forsn(i,1,height-1){ 
		forsn(j,1,width-1){
			acum  = srcd[(i-1)*step +(j-1)]	* m[0]; 	//m[0][0];
			acum += srcd[(i)*step	+(j-1)] 	* m[3];		//m[2][0];
			acum += srcd[(i+1)*step	+(j-1)] 	* m[6];		//m[2][0];
			
			acum += srcd[(i-1)*step	+j] 		* m[1];		//m[0][1];
			acum += srcd[i*step		+j]		* m[4];		//m[1][1];
			acum += srcd[(i+1)*step	+j] 		* m[7];		//m[2][1];
			
			acum += srcd[(i-1)*step +(j+1)]	* m[2];		//m[0][2];
			acum += srcd[i*step		+(j+1)]	* m[5];		//m[1][2];
			acum += srcd[(i+1)*step	+(j+1)]	* m[8];		//m[2][2];

			data[i*step+j]= sat(acum);
		}
	}
	return img;
}


int main( int argc, char** argv )
{
	time_t start,end;
	IplImage  *frame, *outFrame, *showFrame;
	int       key, counter ;
	uchar threshold = 28;
	/* supply the AVI file to play */
	assert( argc == 2 );

	/* load the AVI file */
	CvCapture *capture = cvCaptureFromAVI( argv[1] );

	/* always check */
	if( !capture ) return 1;    

	/* get fps, needed to set the delay */
	int fps = ( int )cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	/* display video */
	cvNamedWindow( "inputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "inputVideo", 10, 10);

	cvNamedWindow( "outputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "outputVideo", 1000, 10);

	time(&start);
	counter = 0;
	int m[9] = {0,1,0,
				1,1,1,
				0,1,0};


	cerr << "threshold = " << threshold;
	frame = cvQueryFrame( capture );
	outFrame = cvCreateImage(cvSize(frame->width, frame->height), frame->depth, 1);
	while( key != 1048689){ //'q' ) {
		/* get a frame */
		frame = cvQueryFrame( capture );

		/* always check */
		if( !frame ) break;

		grayScaleAndThreshold(frame, outFrame, threshold);
		showFrame = convolve(outFrame, m);

		/* display frame */
		cvShowImage( "inputVideo", outFrame );
		cvShowImage( "outputVideo", showFrame );
		cvReleaseImage(&showFrame);

		time(&end);
		++counter;
		double sec = difftime(end,start);
		double fpsRatio = counter/sec;
		cout << fpsRatio << "\t fps " << endl;
		/* quit if user press 'q' */
		key = cvWaitKey( 1000 / fps);
		if(key == 1113938){
			threshold += threshold < 255 ? 1 : 0;
			cerr << "threshold = " << (uint)threshold  << endl;
			key = -1;
			cvWaitKey(1000/fps);
		} else if (key == 1113940){
			threshold -= threshold > 0 ? 1 : 0;
			cerr << "threshold = " << (uint)threshold << endl;
			key = -1;
			cvWaitKey(800/fps);
		}
	}

	cvReleaseImage(&outFrame);
	/* free memory */
	cvReleaseCapture( &capture );
	cvDestroyWindow( "video" );

	return 0;
}
