#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include <time.h>
#include "../lib/orga2.h" 
//The OpenCL C++ bindings, with exceptions
#define __CL_ENABLE_EXCEPTIONS
#include <CL/cl.hpp>
#define KER "./Kernel.cl"

using namespace std;

void cleanUp(){
	cerr << "Cleaning up" << endl;
}


int main(){
	try{
		unsigned int i;
		unsigned int n = 100;
		vector<cl::Platform> platforms;
		cl::Platform::get(&platforms);
		vector<cl::Device> devices;
		platforms.front().getDevices(CL_DEVICE_TYPE_GPU, &devices);
		cl::Context context(devices);
		cl::Device device = devices.front();
		cl::CommandQueue cmdQueue(context, device);

		//kernel	--  aun soporta unicamente un kernel por KER, probablemente no haga falta mas.
		string kernelSrc = convertToString(KER);
		cl::Program::Sources srcCode;
		srcCode.push_back(make_pair(kernelSrc.c_str(), kernelSrc.size()));
		cl::Program program = cl::Program(context, srcCode);
		try{ program.build(devices); }	//build for all devices available
		catch(cl::Error &err){ 
			cerr << "ERROR: Build failed " << err.what() << " (" << err.err() << ")" << endl
				<< "Build log: " << endl
				<< program.getBuildInfo<CL_PROGRAM_BUILD_LOG>(devices.front()) << endl;
			cleanUp();
			exit(1);
		}
		
		cl::Buffer inputBuffer(context, CL_MEM_READ_WRITE, n*sizeof(float));
		cl::Buffer outputBuffer(context, CL_MEM_READ_WRITE, n*sizeof(float));
		//create some data
		vector<float> v_input(n,0);
		vector<float> v_output(n,0);
		forn(i,n) v_input[i] = i*i+0.5;

		cmdQueue.enqueueWriteBuffer(inputBuffer, false, 0, n*sizeof(float), &v_input[0]);	//write buf 2 device main mem
		cmdQueue.enqueueReadBuffer(outputBuffer, false, 0, n*sizeof(float), &v_output[0]);	//read to host mem from dev mem
		cmdQueue.finish();	//wait to finish all the transfers

	} catch (cl::Error& err){
	  clErr(err);
	  cleanUp();
	  return -1;
	}
}
