#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include "../lib/orga2.h" 
//The OpenCL C++ bindings, with exceptions
#define __CL_ENABLE_EXCEPTIONS
#define WORK_GROUP_SIZE_X 16
#define WORK_GROUP_SIZE_Y 16
#define WIDTH 2048 
#define HEIGHT 2048
#define KERFILE "MatrixMul.cl" //source of the kernels
#define KERFUNC "matrixMul" //name of the kernel to be used
#include <CL/cl.hpp>

using namespace std;
bool error = false;

int main(int argc, char** argv){
	unsigned int n, m, l;
	try{
		timerColNames(GPU);
		for(n = m = l = 16; n <= 2048; n *= 2, l*=2, m *=2){
			std::cout << n << "\t";
			unsigned int i;
			vector<cl::Platform> platforms;
			cl::Platform::get(&platforms);
			vector<cl::Device> devices;
			platforms.front().getDevices(CL_DEVICE_TYPE_GPU, &devices);
			cl::Context context(devices);
			cl::Device device = devices.front();

			//load kernel from filename
			std::string kernelSource = convertToString(KERFILE);
			cl::Program::Sources sourceCode;
			sourceCode.push_back(std::make_pair(kernelSource.c_str(), kernelSource.size()));
			
			//create a program
			cl::Program program = cl::Program(context, sourceCode);
			
			//build it for all devices
			try{ program.build(devices); }
			catch(cl::Error &err){
				std::cerr << "Error building kernel " << KERFILE << std::endl;
				tr(devices, it){
					std::cerr << "\t" << it->getInfo<CL_DEVICE_NAME>() << " log: " 
					<< program.getBuildInfo<CL_PROGRAM_BUILD_LOG>(*it) << std::endl;
				}
				return -1;
			}
			cl::CommandQueue cmdQueue(context, device);

			cl::Buffer inputBufferA(context, CL_MEM_READ_WRITE, n*m*sizeof(cl_float));
			cl::Buffer inputBufferB(context, CL_MEM_READ_WRITE, m*l*sizeof(cl_float));
			cl::Buffer outputBuffer(context, CL_MEM_READ_WRITE, n*l*sizeof(cl_float));
			//initialize kernel
			cl::Kernel kernel(program, KERFUNC);
			cl::KernelFunctor kernel_func = kernel.bind(cmdQueue, cl::NDRange(WIDTH, HEIGHT), cl::NDRange(WORK_GROUP_SIZE_X, WORK_GROUP_SIZE_Y));

			float *A = (float*) malloc(sizeof(float) * n * m);
			float *B = (float*) malloc(sizeof(float) * m * l);
			float *AB = (float*) malloc(sizeof(float) * n * l);
			//std::cout << "Running " << NUM_TESTS << " tests" << std::endl;
			timer(INIT);
			forn(i, NUM_TESTS){
				//create some data
				randomizeFloatMatrix((float*) A, n, m);
				randomizeFloatMatrix((float*) B, m, l);

				timer(TOGGLE);
				cmdQueue.enqueueWriteBuffer(inputBufferA, true, 0, n*m*sizeof(cl_float), A);	//write buf 2 device main mem
				cmdQueue.enqueueWriteBuffer(inputBufferB, true, 0, m*l*sizeof(cl_float), B);	//write buf 2 device main mem

				//execute
				kernel_func(inputBufferA, inputBufferB, outputBuffer, m, l);

				cmdQueue.enqueueReadBuffer(outputBuffer, true, 0, n*l*sizeof(cl_float), AB);	//read to host mem from dev mem
				
				timer(TOGGLE);
				#ifdef  TEST	//TODO: hacer el checker prolijo, pero con esto dio bien:
				//check
				/*forn(r, n) forn(j, l){
					float acum = 0;
					forn(k, m) acum += ((float*) (A + r * m))[k] * ((float*) (B + k * l))[j];
					if(abs(((float*)(AB + r*l))[j] - acum) > NUMERICAL_ERROR){
						std::cout << "FAILED " << acum << " \t " << ((float*)(AB+r*l))[j] << std::endl;
					}

				}*/
				//std::cout << std::endl;
				//if(!error) std::cout << "Test: Success (" << NUM_TESTS << " inputs)" << std::endl;
				//else std::cout << "Test: Failed" << std::endl;
				#endif
			}
			timer(FINISH);
		}
	} catch (cl::Error& err){
	  clErr(err);
	  return -1;
	}
}
