#include <stdlib.h>
#include <iostream>
#include <ctime>
#include <cmath>
#include <algorithm>
#include <vector>
#include <cv.h>
#include <highgui.h>

#define forsn(i,s,n) for(i=s;i<n;++i)
#define forn(i,n) forsn(i,0,n)

using namespace std;

void cvPrint(char* str, IplImage* img){
	CvFont font;
	double hScale=1.0;
	double vScale=1.0;
	int    lineWidth=2;
	cvInitFont(&font,CV_FONT_HERSHEY_SIMPLEX|CV_FONT_ITALIC, hScale,vScale,0,lineWidth);
	cvPutText (img,str,cvPoint(40,40), &font, cvScalar(255,255,0));
}

uchar sat(int x){
	if(x<0) return 0;
	else if (x>255) return 255;
	uchar t = x;
	return t;
}

IplImage* noiser(IplImage* src, unsigned int p){	//la probabilidad de que un pixel sea de ruido es ~1/p	
	int height,width,step,channels;
	uchar *data, *srcd;
	int i,j,k;

	// get the image data
	height    = src->height;
	width     = src->width;
	step      = src->widthStep;
	channels  = src->nChannels;
	srcd      = (uchar *)src->imageData;

	IplImage *img = cvCreateImage(cvSize(width, height), src->depth, channels);
	cvCopy(src,img,NULL);

	vector<uchar> v(9);

	data = (uchar *)img->imageData;
	// convolve the image
	forsn(i,1,height-1){ 
		forsn(j,1,width-1){
			if(rand() % p == 0){
				forn(k,channels){
					data[i*step+j*channels+k] = 255;
				}
			} else {
				forn(k,channels){
					data[i*step+j*channels+k] = srcd[i*step+j*channels+k];
				}
			}
		}
	}
	return img;
}


int main( int argc, char** argv )
{
	srand(time(NULL));
	time_t start,end;
	IplImage  *frame, *outFrame;
	int       key, counter ;
	/* supply the AVI file to play */
	if(argc < 3){
		cout << "Usage: ./Noiser <input.avi> <output.avi>" << endl;
		exit(1);
	}

	/* load the AVI file */
	CvCapture *capture = cvCaptureFromAVI( argv[1] );

	/* always check */
	if( !capture ) return 1;    

	/* get fps, needed to set the delay */
	int fps = ( int )cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );

	/* display video */
	cvNamedWindow( "inputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "inputVideo", 10, 10);

	cvNamedWindow( "outputVideo", CV_WINDOW_AUTOSIZE );
	cvMoveWindow( "outputVideo", 1000, 10);

	int height    = cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_HEIGHT);
	int width     = cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_WIDTH);


	CvVideoWriter *writer = cvCreateVideoWriter(argv[2], CV_FOURCC('D', 'I', 'V', 'X'), fps, cvSize(width, height), 1);

	time(&start);
	counter = 0;

	while( key != 1048689){ //'q' ) {
		/* get a frame */
		frame = cvQueryFrame( capture );

		/* always check */
		if( !frame ) break;
		
		outFrame = noiser(frame, 5);

		/* display frame */
		cvShowImage( "inputVideo", frame );
		cvShowImage( "outputVideo", outFrame );
		cvWriteFrame(writer, outFrame);
		cvReleaseImage(&outFrame);
		time(&end);
		++counter;
		double sec = difftime(end,start);
		double fpsRatio = counter/sec;
		cout << fpsRatio << "\t fps " << endl;
		/* quit if user press 'q' */
		key = cvWaitKey( 1000 / fps);
	}

	/* free memory */
	cvReleaseCapture( &capture );
	cvReleaseVideoWriter( &writer );
	cvDestroyWindow( "video" );

	return 0;
}
