#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include "../lib/orga2.h" 
//The OpenCL C++ bindings, with exceptions
#define __CL_ENABLE_EXCEPTIONS
#define KERFILE "Reduce.cl" //source of the kernels
#define KERFUNC "reduce" //name of the kernel to be used
#include <CL/cl.hpp>

using namespace std;
bool error = false;

bool isCorrect(const vector<float> &in, const float result){
	//check if sum in = result
	unsigned int i;
	float res = 0;
	forn(i, in.size()){
		res += in[i];
	}
	//std::cout << std::endl << "should be: " << res << std::endl << "is: " << result << std::endl;
	return abs(res - result)/abs(res) < NUMERICAL_ERROR;
}

void check(const vector<float> &in, const float result){
	if(!isCorrect(in,result)){
		error = true;
		std::cout << "NUMERICAL_ERROR = " << NUMERICAL_ERROR << std::endl; 
		std::cout << "Test failed on input (with result " << result << "): " << std::endl << std::endl;
		printVector(in);
		
		std::cout << std::endl;
	}
}

int main(int argc, char** argv){
	unsigned int n;
	try{
		timerColNames(CPU);
		for(n = VECTOR_STEP; n <= VECTOR_MAX_SIZE; n += VECTOR_STEP){
			std::cout << n << "\t";
			unsigned int i,j;
			vector<cl::Platform> platforms;
			cl::Platform::get(&platforms);
			vector<cl::Device> devices;
			platforms.front().getDevices(CL_DEVICE_TYPE_GPU, &devices);
			cl::Context context(devices);
			cl::Device device = devices.front();

			//load kernel from filename
			std::string kernelSource = convertToString(KERFILE);
			cl::Program::Sources sourceCode;
			sourceCode.push_back(std::make_pair(kernelSource.c_str(), kernelSource.size()));
			
			//create a program
			cl::Program program = cl::Program(context, sourceCode);
			
			//build it for all devices
			try{ program.build(devices); }
			catch(cl::Error &err){
				std::cerr << "Error building kernel " << KERFILE << std::endl;
				tr(devices, it){
					std::cerr << "\t" << it->getInfo<CL_DEVICE_NAME>() << " log: " 
					<< program.getBuildInfo<CL_PROGRAM_BUILD_LOG>(*it) << std::endl;
				}
				return -1;
			}
			cl::CommandQueue cmdQueue(context, device);

			int work_groups = oneDimSize(n)/WORK_GROUP_SIZE;
			//std::cout << " WG = " << work_groups << std::endl;

			cl::Buffer inputBuffer(context, CL_MEM_READ_WRITE, n*sizeof(cl_float));
			cl::Buffer outputBuffer(context, CL_MEM_READ_WRITE, work_groups * sizeof(cl_float));
			//initialize kernel
			cl::Kernel kernel(program, KERFUNC);
			cl::KernelFunctor kernel_func = kernel.bind(cmdQueue, cl::NDRange(oneDimSize(n)), cl::NDRange(WORK_GROUP_SIZE));

			vector<float> v_input(n,0);
			vector<float> output(work_groups, 0);
			timer(INIT);
			forn(i, NUM_TESTS){
				//create some data
				randomizeFloatVector(v_input);
				timer(TOGGLE);
				forn(j, output.size()) output[j] = 0;
				forn(j, v_input.size())	output[j%work_groups] += v_input[j];

				float sumOut = 0;
				tr(output, it){
					sumOut += *it;
				}
				timer(TOGGLE);
				#ifdef TEST
				check(v_input, sumOut);
				std::cout << std::endl;
				if(!error) std::cout << "Test: Success (" << NUM_TESTS << " inputs)" << std::endl;
				else std::cout << "Test: Failed" << std::endl;
				#endif
			}
			timer(FINISH);
		}
	} catch (cl::Error& err){
	  clErr(err);
	  return -1;
	}
}
