#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>
#include <cmath>
#include <time.h>
#include "../lib/orga2.h" 

using namespace std;
bool error = false;

bool isCorrect(const vector<float> &a, const vector<float> &b, const vector<float> &result){
	//check if a+b = c
	unsigned int i;
	if(a.size() != b.size()) return false;
	if(b.size() != result.size()) return false;
	forn(i, a.size()){
		if(a[i] + b[i] != result[i]) return false;
	}
	return true;
}

void check(const vector<float> &a, const vector<float> &b, const vector<float> &result){
	if(!isCorrect(a,b,result)){
		error = true;
		std::cout << "Test failed on input: " << std::endl;
		printVector(a);
		std::cout << std::endl;
		printVector(b);
		std::cout << std::endl;
	}
}

int main(int argc, char** argv){
	unsigned int n;
	try{
		timerColNames(CPU);
		for(n = VECTOR_STEP; n < VECTOR_MAX_SIZE; n+=VECTOR_STEP){
			std::cout << n << "\t";
			unsigned int i;
			vector<float> v_inputA(n,0);
			vector<float> v_inputB(n,0);
			vector<float> v_output(n,0);
			timer(INIT);
			forn(i, NUM_TESTS){
				//create some data
				randomizeFloatVector(v_inputA);
				randomizeFloatVector(v_inputB);
				timer(TOGGLE);
				forn(i, v_inputA.size()) v_output[i] = v_inputA[i] + v_inputB[i];
				#ifdef TEST
				check(v_inputA, v_inputB, v_output);
				if(!error) std::cout << "Test: Success (" << NUM_TESTS << " inputs)" << std::endl;
				else std::cout << "Test: Failed" << std::endl;
				#endif
				timer(TOGGLE);
			}
			timer(FINISH);
		}
	} catch (cl::Error& err){
	  clErr(err);
	  return -1;
	}
}
