#include "orga2.h"
#include <cstdlib>
#include <ctime>

bool randomSeed = false;
void clErr(cl::Error& err){
	std::cerr << "OpenCL: " << err.what() << " - " << errorString(err.err()) << std::endl;
}

void checkErr(cl_int err, const char * errStr){
	if (err != CL_SUCCESS) {
		std::cout << std::endl;
        std::cerr << "\nERROR: " << errStr << " (" << errorString(err) << ")" << std::endl;
        exit(EXIT_FAILURE);
    }
}

// error strings
const char* errorString(cl_int error)
{
    static const char* errorString[] = {
        "CL_SUCCESS",
        "CL_DEVICE_NOT_FOUND",
        "CL_DEVICE_NOT_AVAILABLE",
        "CL_COMPILER_NOT_AVAILABLE",
        "CL_MEM_OBJECT_ALLOCATION_FAILURE",
        "CL_OUT_OF_RESOURCES",
        "CL_OUT_OF_HOST_MEMORY",
        "CL_PROFILING_INFO_NOT_AVAILABLE",
        "CL_MEM_COPY_OVERLAP",
        "CL_IMAGE_FORMAT_MISMATCH",
        "CL_IMAGE_FORMAT_NOT_SUPPORTED",
        "CL_BUILD_PROGRAM_FAILURE",
        "CL_MAP_FAILURE",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "",
        "CL_INVALID_VALUE",
        "CL_INVALID_DEVICE_TYPE",
        "CL_INVALID_PLATFORM",
        "CL_INVALID_DEVICE",
        "CL_INVALID_CONTEXT",
        "CL_INVALID_QUEUE_PROPERTIES",
        "CL_INVALID_COMMAND_QUEUE",
        "CL_INVALID_HOST_PTR",
        "CL_INVALID_MEM_OBJECT",
        "CL_INVALID_IMAGE_FORMAT_DESCRIPTOR",
        "CL_INVALID_IMAGE_SIZE",
        "CL_INVALID_SAMPLER",
        "CL_INVALID_BINARY",
        "CL_INVALID_BUILD_OPTIONS",
        "CL_INVALID_PROGRAM",
        "CL_INVALID_PROGRAM_EXECUTABLE",
        "CL_INVALID_KERNEL_NAME",
        "CL_INVALID_KERNEL_DEFINITION",
        "CL_INVALID_KERNEL",
        "CL_INVALID_ARG_INDEX",
        "CL_INVALID_ARG_VALUE",
        "CL_INVALID_ARG_SIZE",
        "CL_INVALID_KERNEL_ARGS",
        "CL_INVALID_WORK_DIMENSION",
        "CL_INVALID_WORK_GROUP_SIZE",
        "CL_INVALID_WORK_ITEM_SIZE",
        "CL_INVALID_GLOBAL_OFFSET",
        "CL_INVALID_EVENT_WAIT_LIST",
        "CL_INVALID_EVENT",
        "CL_INVALID_OPERATION",
        "CL_INVALID_GL_OBJECT",
        "CL_INVALID_BUFFER_SIZE",
        "CL_INVALID_MIP_LEVEL",
        "CL_INVALID_GLOBAL_WORK_SIZE",
    };
    const int errorCount = sizeof(errorString) / sizeof(errorString[0]);
    const int index = -error;
    return (index >= 0 && index < errorCount) ? errorString[index] : "";
}

void showIntMatrix(int n, int m, int *A){
	int i,j;
	forn(i, n){
		forn(j,m){
			printf("%3d ", *(A+(m*i)+j));
		}
		std::cout << std::endl;
	}
}

void showFloatMatrix(float *A, unsigned int n, unsigned int m){
	unsigned int i,j;
	forn(i, n){
		forn(j,m){
			printf("%3f ", *(A+(m*i)+j));
		}
		std::cout << std::endl;
	}
}

std::string convertToString(const char *filename){
	size_t size;
	char*  str;
	std::string s;

	std::fstream f(filename, (std::fstream::in | std::fstream::binary));

	if(f.is_open()){
		size_t fileSize;
		f.seekg(0, std::fstream::end);
		size = fileSize = (size_t)f.tellg();
		f.seekg(0, std::fstream::beg);

		str = new char[size+1];
		if(!str){
			f.close();
			return NULL;
		}

		f.read(str, fileSize);
		f.close();
		str[size] = '\0';
	
		s = str;
		delete[] str;
		return s;
	}else{
		std::cerr << "\nFile " << filename << " not found.\n";
		exit(1);
	}
	return NULL;
}

unsigned int oneDimSize(int n){
	if(n % WORK_GROUP_SIZE == 0) return n;
	return ((n / WORK_GROUP_SIZE)+1)*WORK_GROUP_SIZE;
}
void randomizeIntegerVector(std::vector<int> &v){
	if(!randomSeed) srand((unsigned) time(0));
	randomSeed = true;
	unsigned int i;
	forn(i, v.size()) v[i] = rand();
}

void randomizeFloatVector(std::vector<float> &v){
	if(!randomSeed) srand((unsigned) time(0));
	randomSeed = true;
	unsigned int i;
	forn(i, v.size()) v[i] = (float) rand()/RAND_MAX;
}

void randomizeIntegerMatrix(std::vector<std::vector<int> > &m){
	
}
void randomizeFloatMatrix(float *M, unsigned int n, unsigned int m){
	if(!randomSeed) srand((unsigned) time(0));
	randomSeed = true;
	unsigned int i,j;
	forn(i, n)
		forn(j, m) 
			((float*)(M+i*m))[j] = (float) rand()/RAND_MAX;
}

void timerColNames(colType ct){	//para plotter 
	std::cout << "Problem Size\t";
	if( ct == GPU) std::cout << "[GPU] Avg Time";
	else std::cout << "[CPU] Avg Time";
	std::cout << std::endl;
}

void timer(timerOp op){
	static timeval lastTime, curTime;
	static double sum;
	static unsigned int n;
	static bool counting;
	if(op == INIT){
		n = 0;
		sum = 0;
		counting = false;
	}
	else if(op == TOGGLE){
		if(counting){
			gettimeofday(&curTime, NULL);
			double tS = lastTime.tv_sec*1000000 + lastTime.tv_usec;
			double tE = curTime.tv_sec*1000000 + curTime.tv_usec;
			sum += tE - tS;
			n++;
			counting = false;
		}
		else{
			counting = true;
			gettimeofday(&lastTime, NULL);
		}
	}
	else if(op == FINISH){
		if(counting) std::cout << "WARNING: toggle incorrect" << std::endl;
		std::cout << std::setprecision(10000)  << sum/n << std::endl;
	}
}
